@@ -204,9 +204,9 @@ bool is_cntrl_char(const char *c);
 bool is_word_char(const char *c, bool allow_punct);
 char control_mbrep(const char *c, bool isdata);
 #ifdef ENABLE_UTF8
-int mbwidth(const char *c);
+int mbtowide(wchar_t *wc, const char *c);
+bool is_doublewidth(const char *ch);
 bool is_zerowidth(const char *ch);
-char *make_mbchar(long code, int *length);
 #endif
 int char_length(const char *pointer);
 size_t mbstrlen(const char *pointer);
@@ -230,16 +230,12 @@ char *mbrevstrpbrk(const char *head, const char *accept, const char *pointer);
 bool has_blank_char(const char *string);
 #endif
 bool white_string(const char *string);
-#ifdef ENABLE_UTF8
-bool is_valid_unicode(wchar_t wc);
-#endif
 
 /* Most functions in color.c. */
 #ifdef ENABLE_COLOR
 void set_interface_colorpairs(void);
 void prepare_palette(void);
 void find_and_prime_applicable_syntax(void);
-void set_up_multicache(linestruct *line);
 void check_the_multis(linestruct *line);
 void precalc_multicolorinfo(void);
 #endif
@@ -413,7 +409,6 @@ void block_sigwinch(bool blockit);
 #endif
 #ifndef NANO_TINY
 void handle_sigwinch(int signal);
-void compute_the_extra_rows_per_line_from(linestruct *fromline);
 void regenerate_screen(void);
 void do_toggle(int flag);
 #endif
@@ -512,8 +507,7 @@ size_t indent_length(const char *line);
 size_t quote_length(const char *line);
 bool begpar(const linestruct *const line, int depth);
 bool inpar(const linestruct *const line);
-void do_justify(bool full_justify);
-void do_justify_void(void);
+void do_justify(void);
 void do_full_justify(void);
 #endif
 #ifdef ENABLE_SPELLER
@@ -533,9 +527,7 @@ void complete_a_word(void);
 void get_homedir(void);
 const char *tail(const char *path);
 char *concatenate(const char *path, const char *name);
-#ifdef ENABLE_LINENUMBERS
 int digits(ssize_t n);
-#endif
 bool parse_num(const char *str, ssize_t *result);
 bool parse_line_column(const char *str, ssize_t *line, ssize_t *column);
 void recode_NUL_to_LF(char *string, size_t length);
