@@ -60,8 +60,8 @@ void do_mark(void)
 }
 #endif /* !NANO_TINY */
 
-/* Insert a tab.  If the TABS_TO_SPACES flag is set, insert the number
- * of spaces that a tab would normally take up. */
+/* Insert a tab.  Or, if --tabstospaces is in effect, insert the number
+ * of spaces that a tab would normally take up at this position. */
 void do_tab(void)
 {
 #ifdef ENABLE_COLOR
@@ -103,9 +103,6 @@ void indent_a_line(linestruct *line, char *indentation)
 
 	openfile->totsize += indent_len;
 
-	if (ISSET(SOFTWRAP))
-		line->extrarows = extra_chunks_in(line);
-
 	/* Compensate for the change in the current line. */
 	if (line == openfile->mark && openfile->mark_x > 0)
 		openfile->mark_x += indent_len;
@@ -136,12 +133,12 @@ void do_indent(void)
 
 	indentation = nmalloc(tabsize + 1);
 
-	/* Set the indentation to either a bunch of spaces or a single tab. */
 #ifdef ENABLE_COLOR
 	if (openfile->syntax && openfile->syntax->tab)
 		indentation = mallocstrcpy(indentation, openfile->syntax->tab);
 	else
 #endif
+	/* Set the indentation to either a bunch of spaces or a single tab. */
 	if (ISSET(TABS_TO_SPACES)) {
 		memset(indentation, ' ', tabsize);
 		indentation[tabsize] = '\0';
@@ -234,9 +231,6 @@ void unindent_a_line(linestruct *line, size_t indent_len)
 
 	openfile->totsize -= indent_len;
 
-	if (ISSET(SOFTWRAP))
-		line->extrarows = extra_chunks_in(line);
-
 	/* Adjust the positions of mark and cursor, when they are affected. */
 	compensate_leftward(line, indent_len);
 }
@@ -284,9 +278,6 @@ void handle_indent_action(undostruct *u, bool undoing, bool add_indent)
 	groupstruct *group = u->grouping;
 	linestruct *line = line_from_number(group->top_line);
 
-	if (group->next != NULL)
-		die("Multiple groups -- please report a bug\n");
-
 	/* When redoing, reposition the cursor and let the indenter adjust it. */
 	if (!undoing)
 		goto_line_posx(u->head_lineno, u->head_x);
@@ -424,15 +415,9 @@ void do_comment(void)
 
 	/* Comment/uncomment each of the selected lines when possible, and
 	 * store undo data when a line changed. */
-	for (line = top; line != bot->next; line = line->next) {
-		if (comment_line(action, line, comment_seq)) {
-#ifndef NANO_TINY
-			if (ISSET(SOFTWRAP))
-				line->extrarows = extra_chunks_in(line);
-#endif
+	for (line = top; line != bot->next; line = line->next)
+		if (comment_line(action, line, comment_seq))
 			update_multiline_undo(line->lineno, "");
-		}
-	}
 
 	set_modified();
 	ensure_firstcolumn_is_aligned();
@@ -573,8 +558,6 @@ void do_undo(void)
 			break;
 		}
 		line->data[u->tail_x] = '\0';
-		if (ISSET(SOFTWRAP))
-			line->extrarows = extra_chunks_in(line);
 		intruder = make_new_node(line);
 		intruder->data = copy_of(u->strdata);
 		splice_node(line, intruder);
@@ -676,9 +659,6 @@ void do_undo(void)
 	openfile->mark = NULL;
 	openfile->placewewant = xplustabs();
 
-	if (ISSET(SOFTWRAP))
-		openfile->current->extrarows = extra_chunks_in(openfile->current);
-
 	openfile->totsize = u->wassize;
 
 	/* When at the point where the file was last saved, unset "Modified". */
@@ -725,8 +705,6 @@ void do_redo(void)
 	case ENTER:
 		redidmsg = _("line break");
 		line->data[u->head_x] = '\0';
-		if (ISSET(SOFTWRAP))
-			line->extrarows = extra_chunks_in(line);
 		intruder = make_new_node(line);
 		intruder->data = copy_of(u->strdata);
 		splice_node(line, intruder);
@@ -841,9 +819,6 @@ void do_redo(void)
 	openfile->mark = NULL;
 	openfile->placewewant = xplustabs();
 
-	if (ISSET(SOFTWRAP))
-		openfile->current->extrarows = extra_chunks_in(openfile->current);
-
 	openfile->totsize = u->newsize;
 
 	/* When at the point where the file was last saved, unset "Modified". */
@@ -907,11 +882,6 @@ void do_enter(void)
 		openfile->mark = newnode;
 		openfile->mark_x += extra - openfile->current_x;
 	}
-
-	if (ISSET(SOFTWRAP)) {
-		openfile->current->extrarows = extra_chunks_in(openfile->current);
-		newnode->extrarows = extra_chunks_in(newnode);
-	}
 #endif
 
 	/* Insert the newly created line after the current one and renumber. */
@@ -951,7 +921,7 @@ void discard_until(const undostruct *thisitem)
 		while (group != NULL) {
 			groupstruct *next = group->next;
 			free_chararray(group->indentations,
-								group->bottom_line - group->top_line);
+								group->bottom_line - group->top_line + 1);
 			free(group);
 			group = next;
 		}
@@ -1112,24 +1082,24 @@ void update_multiline_undo(ssize_t lineno, char *indentation)
 	/* If there already is a group and the current line is contiguous with it,
 	 * extend the group; otherwise, create a new group. */
 	if (u->grouping && u->grouping->bottom_line + 1 == lineno) {
-		size_t number_of_lines;
+		size_t number_of_lines = lineno - u->grouping->top_line + 1;
 
-		u->grouping->bottom_line++;
+		u->grouping->bottom_line = lineno;
 
-		number_of_lines = u->grouping->bottom_line - u->grouping->top_line + 1;
 		u->grouping->indentations = nrealloc(u->grouping->indentations,
 										number_of_lines * sizeof(char *));
 		u->grouping->indentations[number_of_lines - 1] = copy_of(indentation);
 	} else {
 		groupstruct *born = nmalloc(sizeof(groupstruct));
 
-		born->next = u->grouping;
-		u->grouping = born;
 		born->top_line = lineno;
 		born->bottom_line = lineno;
 
-		u->grouping->indentations = nmalloc(sizeof(char *));
-		u->grouping->indentations[0] = copy_of(indentation);
+		born->indentations = nmalloc(sizeof(char *));
+		born->indentations[0] = copy_of(indentation);
+
+		born->next = u->grouping;
+		u->grouping = born;
 	}
 
 	/* Store the file size after the change, to be used when redoing. */
@@ -1357,7 +1327,9 @@ bool do_wrap(void)
 		strncpy(line->data, line->prev->data, lead_len);
 
 		openfile->current_x += lead_len;
+		openfile->totsize += lead_len;
 #ifndef NANO_TINY
+		free(openfile->undotop->strdata);
 		update_undo(ENTER);
 #endif
 		if (autowhite)
@@ -1453,21 +1425,12 @@ ssize_t break_line(const char *textstart, ssize_t goal, bool snap_at_nl)
  * "indentation" of a line is the leading consecutive whitespace. */
 size_t indent_length(const char *line)
 {
-	size_t len = 0;
-	char onechar[MAXCHARLEN];
-	int charlen;
+	const char *start = line;
 
-	while (*line != '\0') {
-		charlen = collect_char(line, onechar);
+	while (*line != '\0' && is_blank_char(line))
+		line += char_length(line);
 
-		if (!is_blank_char(onechar))
-			break;
-
-		line += charlen;
-		len += charlen;
-	}
-
-	return len;
+	return (line - start);
 }
 #endif
 
@@ -1741,9 +1704,12 @@ void justify_paragraph(linestruct **line, size_t count)
 	free(lead_string);
 }
 
-/* Justify the current paragraph, or the entire buffer when full_justify is
+#define ONE_PARAGRAPH  FALSE
+#define WHOLE_BUFFER  TRUE
+
+/* Justify the current paragraph, or the entire buffer when whole_buffer is
  * TRUE.  But if the mark is on, justify only the marked text instead. */
-void do_justify(bool full_justify)
+void justify_text(bool whole_buffer)
 {
 	size_t linecount;
 		/* The number of lines in the original paragraph. */
@@ -1859,7 +1825,7 @@ void do_justify(bool full_justify)
 	{
 		/* When justifying the entire buffer, start at the top.  Otherwise, when
 		 * in a paragraph but not at its beginning, move back to its first line. */
-		if (full_justify)
+		if (whole_buffer)
 			openfile->current = openfile->filetop;
 		else if (inpar(openfile->current) && !begpar(openfile->current, 0))
 			do_para_begin(&openfile->current);
@@ -1881,7 +1847,7 @@ void do_justify(bool full_justify)
 		start_x = 0;
 
 		/* Set the end point of the paragraph. */
-		if (full_justify)
+		if (whole_buffer)
 			endline = openfile->filebot;
 		else {
 			endline = startline;
@@ -1956,7 +1922,7 @@ void do_justify(bool full_justify)
 		justify_paragraph(&jusline, linecount);
 
 		/* When justifying the entire buffer, find and justify all paragraphs. */
-		if (full_justify) {
+		if (whole_buffer) {
 			while (find_paragraph(&jusline, &linecount)) {
 				justify_paragraph(&jusline, linecount);
 
@@ -1968,7 +1934,7 @@ void do_justify(bool full_justify)
 
 #ifndef NANO_TINY
 	add_undo(PASTE, NULL);
-	if (full_justify && !openfile->mark && !cutbuffer->has_anchor)
+	if (whole_buffer && !openfile->mark && !cutbuffer->has_anchor)
 		openfile->current->has_anchor = FALSE;
 #endif
 	/* Do the equivalent of a paste of the justified text. */
@@ -1999,8 +1965,8 @@ void do_justify(bool full_justify)
 		statusline(REMARK, _("Justified selection"));
 	else
 #endif
-	if (full_justify)
-		statusbar(_("Justified file"));
+	if (whole_buffer)
+		statusline(REMARK, _("Justified file"));
 	else
 		statusbar(_("Justified paragraph"));
 
@@ -2013,15 +1979,15 @@ void do_justify(bool full_justify)
 }
 
 /* Justify the current paragraph. */
-void do_justify_void(void)
+void do_justify(void)
 {
-	do_justify(FALSE);
+	justify_text(ONE_PARAGRAPH);
 }
 
 /* Justify the entire file. */
 void do_full_justify(void)
 {
-	do_justify(TRUE);
+	justify_text(WHOLE_BUFFER);
 	ran_a_tool = TRUE;
 }
 #endif /* ENABLE_JUSTIFY */
@@ -2644,6 +2610,9 @@ void do_linter(void)
 		return;
 	}
 
+	/* Block resizing signals while reading from the pipe. */
+	block_sigwinch(TRUE);
+
 	/* Read in the returned syntax errors. */
 	totalread = 0;
 	buffersize = pipesize + 1;
@@ -2660,10 +2629,12 @@ void do_linter(void)
 	*pointer = '\0';
 	close(lint_fd[0]);
 
-	/* Process the linter output. */
+	block_sigwinch(FALSE);
+
 	pointer = lintings;
 	onelint = lintings;
 
+	/* Now parse the output of the linter. */
 	while (*pointer != '\0') {
 		if ((*pointer == '\r') || (*pointer == '\n')) {
 			*pointer = '\0';
@@ -2671,11 +2642,8 @@ void do_linter(void)
 				char *filename = NULL, *linestr = NULL, *maybecol = NULL;
 				char *message = copy_of(onelint);
 
-				/* At the moment we handle the following formats:
-				 *
-				 * filenameorcategory:line:column:message (e.g. splint)
-				 * filenameorcategory:line,column:message (e.g. pylint)
-				 * filenameorcategory:line:message        (e.g. pyflakes) */
+				/* The recognized format is "filename:line:column: message",
+				 * where ":column" may be absent or be ",column" instead. */
 				if (strstr(message, ": ") != NULL) {
 					filename = strtok(onelint, ":");
 					if ((linestr = strtok(NULL, ":")) != NULL) {
@@ -2708,7 +2676,7 @@ void do_linter(void)
 							curlint->prev = tmplint;
 							if (curlint->prev != NULL)
 								curlint->prev->next = curlint;
-							curlint->msg = copy_of(message);
+							curlint->msg = copy_of(strstr(message, ": ") + 2);
 							curlint->lineno = tmplineno;
 							curlint->colno = tmpcolno;
 							curlint->filename = copy_of(filename);
@@ -3193,8 +3161,8 @@ void complete_a_word(void)
 
 	/* The search has reached the end of the file. */
 	if (list_of_completions != NULL) {
+		edit_refresh();
 		statusline(AHEM, _("No further matches"));
-		refresh_needed = TRUE;
 	} else
 		/* TRANSLATORS: Shown when there are zero possible completions. */
 		statusline(AHEM, _("No matches"));
